#!/usr/bin/env python
#coding: utf-8

# gui.py - GTK3 GUI for SAPI Book Reader
# Copyright (C) Bohdan R. Rau 2013 <ethanak@polip.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.


from gi.repository import Gtk,GtkSource,Pango,GLib,GObject,Gdk
import re,os

class gui(Gtk.Window):
    @staticmethod
    def exit(*args):
        try:
            pass
        except:
            pass
        from ctypes import CDLL
        CDLL("libc.so.6").exit(0)

    def place_cursor(self, pos):
        self._cursor=pos
        GLib.timeout_add(300,self._init_cursor,None)

    def set_text(self, text,n=0):
        self.buffer.set_text(text)
        self._cursor=n
        GLib.timeout_add(300,self._init_cursor,None)
        
    def alert(self,txt):
        dialog=Gtk.MessageDialog(self,3,Gtk.MessageType.INFO,Gtk.ButtonsType.OK,txt)
        dialog.run()
        dialog.destroy()
    
    def ask(self,txt):
        dialog=Gtk.MessageDialog(self,3,Gtk.MessageType.QUESTION,Gtk.ButtonsType.YES_NO,txt)
        rc=dialog.run()
        dialog.destroy()
        return rc==Gtk.ResponseType.YES
    
    def loop(self):
        Gtk.main()
    
    def set_play_position(self, pos,count):
        self.position=(pos,count)
        
    def end_of_book(self):
        self.speaking=False
    
    def query(self,what,title='Question'):
        ls=[]
        dialog=Gtk.Dialog(title,self,3)
        dialog.add_button(Gtk.STOCK_CANCEL,0)
        dialog.add_button(Gtk.STOCK_OK,1)
        inn=dialog.get_content_area()
        grid=Gtk.Grid()
        inn.pack_start(grid,0,0,2)
        for i,a in enumerate(what):
            if isinstance(a,str) or isinstance(a,unicode):
                label=a
                key=a
            else:
                label=a[0]
                key=a[1]
            lb=Gtk.Label(label)
            grid.attach(lb,0,i,1,1)
            entry=Gtk.Entry()
            entry.set_size_request(250,-1)
            grid.attach(entry,1,i,1,1)
            ls.append((key,entry))
        inn.show_all()
        r=dialog.run()
        rc=None
        if r == 1:
            rc={}
            for a in ls:
                rc[a[0]]=a[1].get_text()
        dialog.destroy()
        return rc
            

        
    
    # private
    
    
    def _apply_font(self):
        if self.font:
            font_descr=Pango.font_description_from_string(self.font)
            self.view.modify_font(font_descr)
            r=re.match(r'.*\s([0-9]+)$',self.font)
            if r:
                n=int(r.group(1)) /2
                self.view.set_pixels_below_lines(n)
                
            
            
    
    def _select_font(self,*args):
        if self.speaking:
            return
        dialog=Gtk.FontChooserDialog('Select font',self)
        if self.font:
            dialog.set_font(self.font)
        i1=self.buffer.get_start_iter()
        i2=self.buffer.get_iter_at_offset(80)
        txt=self.buffer.get_text(i1,i2,False).strip()
        txt=re.sub(r'\s+',' ',txt)
        if len(txt) > 16:
            dialog.set_preview_text(txt)
        r=dialog.run()
        if r == Gtk.ResponseType.OK:
            self.font=dialog.get_font()
            self._apply_font()
            self.core.modify_params({'font':self.font})
        dialog.destroy()
    
    
    def _translate(self,*args):
        if self.speaking:
            return
        txt=''
        sel=self.buffer.get_selection_bounds()
        if sel:
            txt=self.buffer.get_text(sel[0],sel[1],False)
        else:
            txt=self._last_txt
        langs=self.core.translator_languages()
        if not langs:
            return
        dialog=Gtk.Dialog('Translator',self,3)
        dialog.add_button(Gtk.STOCK_CANCEL,0)
        dialog.add_button(Gtk.STOCK_OK,1)
        inn=dialog.get_content_area()
        hb=Gtk.Box(Gtk.Orientation.HORIZONTAL)
        inn.pack_start(hb,0,0,2)
        entry=Gtk.Entry()
        entry.set_text(txt)
        entry.set_size_request(750,-1);
        hb.pack_start(entry,0,0,2)
        hb=Gtk.Box(Gtk.Orientation.HORIZONTAL)
        inn.pack_start(hb,0,0,2)
        lfr=Gtk.ComboBoxText()
        for a in langs:
            lfr.append_text(a[0])
        hb.pack_start(lfr,0,0,2) 
        lto=Gtk.ComboBoxText()
        for a in langs:
            lto.append_text(a[0])
        hb.pack_start(Gtk.Label('=>'),0,0,2) 
        hb.pack_start(lto,0,0,2)
        l1=0
        l2=0
        for n,a in enumerate(langs):
            if a[1] == self.llang_f:
                l1=n
            if a[1] == self.llang_t:
                l2=n
        lfr.set_active(l1)
        lto.set_active(l2)
        inn.show_all()
        r=dialog.run()
        txt=entry.get_text()
        l1=lfr.get_active()
        l2=lto.get_active()
        dialog.destroy()
        if r != 1:
            return None
        self.llang_f=langs[l1][1]
        self.llang_t=langs[l2][1]
        self.core.modify_params({'langf':self.llang_f,'langt':self.llang_t})
        if not txt:
            return None
        self._last_txt=txt
        txt=self.core.translate(txt,self.llang_f,self.llang_t)
        return txt
        
    
    
    def _open_file(self,*args):
        if self.speaking:
            return
        dialog=Gtk.FileChooserDialog('Open file',self,Gtk.FileChooserAction.OPEN,
                                     (Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OK,Gtk.ResponseType.OK))
        dialog.set_local_only(True)
        dialog.set_current_folder(self._last_dir)
        f1=Gtk.FileFilter()
        f1.set_name('Text files')
        f1.add_pattern('*.txt')
        dialog.add_filter(f1)
        f2=Gtk.FileFilter()
        f2.set_name('All files')
        f2.add_pattern('*')
        dialog.add_filter(f2)
        dialog.set_filter(f1)
        rc=dialog.run()
        fname=None
        if rc == Gtk.ResponseType.OK:
            fname=dialog.get_filename()
            self._last_dir=dialog.get_current_folder()
        dialog.destroy()
        if (fname):
            self.core.set_file(fname)
    
    def _speak_start(self,*args):
        if self.speaking:
            return
        lang=self.voices[self.voicecombo.get_active()][1]
        vox=self.voices[self.voicecombo.get_active()][0]
        speed=self.voicespeed.get_value()
        self.core.modify_params({
            'voice':vox,
            'speed':speed})
        tm=self.buffer.get_insert()
        itr1=self.buffer.get_iter_at_mark(tm)
        itr2=self.buffer.get_end_iter()
        offset=itr1.get_offset()
        txt=self.buffer.get_text(itr1,itr2,False).decode('utf-8')
        if not self.core.init_speaker(vox,lang,offset,txt,speed):
            return
        self.speaking=True
        self.start_button.set_sensitive(False)
        self.stop_button.set_sensitive(True)
        GLib.timeout_add(100,self._show_speak,None)
        self.last_cursor_pos=offset-1;
        self.position=None
        self.old_position=None
        self.core.start_speak()

    def _is_position_visible(self):
        r=self.view.get_visible_rect()
        t=self.view.get_iter_location(self.buffer.get_iter_at_offset(self.position[0]))
        if t.y < r.y:
            return True
        t=self.view.get_iter_location(self.buffer.get_iter_at_offset(self.position[0]+self.position[1]))
        return t.y+t.height > r.y+r.height
            
    
    def _show_speak(self, *args):
        if not self.speaking:
            if (self.old_position):
                self.buffer.remove_tag(self.speaktag,
                                       self.buffer.get_iter_at_offset(self.old_position[0]),
                                       self.buffer.get_iter_at_offset(self.old_position[0]+self.old_position[1]))
            self.start_button.set_sensitive(True)
            self.stop_button.set_sensitive(False)
            return False
        if self.position and self.position[0] != self.last_cursor_pos:
            self.last_cursor_pos=self.position[0]
            itr=self.buffer.get_iter_at_offset(self.last_cursor_pos)
            if self._is_position_visible():
                self.view.scroll_to_iter(itr,0.05,True,0.1,0.0)
            self.buffer.place_cursor(itr)
            if (self.old_position):
                self.buffer.remove_tag(self.speaktag,
                                       self.buffer.get_iter_at_offset(self.old_position[0]),
                                       self.buffer.get_iter_at_offset(self.old_position[0]+self.old_position[1]))
            self.old_position=self.position
            self.buffer.apply_tag(self.speaktag,
                                       self.buffer.get_iter_at_offset(self.old_position[0]),
                                       self.buffer.get_iter_at_offset(self.old_position[0]+self.old_position[1]))
            
            
        return True
    
    def _clauth(self, *args):
        if not self.ask('Clear MS client id and secret?'):
            return
        self.core.modify_params({'ms_client_id':None,'ms_client_secret':None})
    
    def _speak_stop(self,*args):
        self.core.stop_speak()
        self.speaking=False
        self.start_button.set_sensitive(True)
        self.stop_button.set_sensitive(False)
    
    def __init__(self,core):
        Gtk.Window.__init__(self)
        self.core=core
        self.error=None
        self._last_txt=''
        self._last_dir=os.getcwd()
        params=self.core.get_params(['font','langf','langt','voice','speed'])
        self.font=params['font']
        self.llang_f=params['langf']
        self.llang_t=params['langt']
        self.speaking=False
        
        
        self.cid=self.connect("destroy",self.exit)
        frame=Gtk.VBox(0,2)
        self.add(frame)
        
        accel=Gtk.AccelGroup()
        self.add_accel_group (accel)

        menubar=Gtk.MenuBar()
        filemenu=Gtk.Menu()
        speechmenu=Gtk.Menu()
        transmenu=Gtk.Menu()
        
        m_file=Gtk.MenuItem.new_with_mnemonic('_File')
        m_file.set_submenu(filemenu)
        m_open=Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_OPEN,None)
        m_open.add_accelerator('activate',accel,Gdk.KEY_o,Gdk.ModifierType.CONTROL_MASK,Gtk.AccelFlags.VISIBLE)
        m_open.connect('activate',self._open_file,None)
        m_font=Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_SELECT_FONT,None)
        m_font.connect('activate',self._select_font,None)
        m_quit=Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_QUIT,None)
        m_quit.add_accelerator('activate',accel,Gdk.KEY_q,Gdk.ModifierType.CONTROL_MASK,Gtk.AccelFlags.VISIBLE)
        m_quit.connect('activate',self.exit,None)
        filemenu.append(m_open)
        filemenu.append(m_font)
        filemenu.append(m_quit)
        menubar.append(m_file)

        m_speech=Gtk.MenuItem.new_with_mnemonic('_Speech')
        m_speech.set_submenu(speechmenu)
        m_play=Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_MEDIA_PLAY,None)
        m_play.add_accelerator('activate',accel,Gdk.KEY_p,Gdk.ModifierType.CONTROL_MASK,Gtk.AccelFlags.VISIBLE)
        m_play.connect('activate',self._speak_start,None)
        m_stop=Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_MEDIA_STOP,None)
        m_stop.add_accelerator('activate',accel,Gdk.KEY_Escape,0,Gtk.AccelFlags.VISIBLE)
        m_stop.connect('activate',self._speak_stop,None)
        speechmenu.append(m_play)
        speechmenu.append(m_stop)
        menubar.append(m_speech)

        m_trans=Gtk.MenuItem.new_with_mnemonic('T_ranslator')
        m_trans.set_submenu(transmenu)
        m_tran=Gtk.MenuItem('Translate')
        m_tran.add_accelerator('activate',accel,Gdk.KEY_t,Gdk.ModifierType.CONTROL_MASK,Gtk.AccelFlags.VISIBLE)
        m_tran.connect('activate',self._translate,None)
        m_tcln=Gtk.MenuItem('Clean auth data')
        m_tcln.connect('activate',self._clauth,None)
        transmenu.append(m_tran)
        transmenu.append(m_tcln)
        menubar.append(m_trans)

        frame.pack_start(menubar,0,0,3)
        
        self.toolbar=Gtk.Toolbar()
        self.toolbar.set_style(Gtk.ToolbarStyle.BOTH)
        frame.pack_start(self.toolbar,0,0,0)
        
        button=Gtk.ToolButton.new_from_stock(Gtk.STOCK_OPEN)
        self.toolbar.insert(button,-1)
        button.connect('clicked',self._open_file,None)

        self.toolbar.insert(Gtk.SeparatorToolItem(),-1)

        self.start_button=Gtk.ToolButton.new_from_stock(Gtk.STOCK_MEDIA_PLAY)
        self.toolbar.insert(self.start_button,-1)
        self.start_button.connect('clicked',self._speak_start,None)
        self.stop_button=Gtk.ToolButton.new_from_stock(Gtk.STOCK_MEDIA_STOP)
        self.toolbar.insert(self.stop_button,-1)
        self.stop_button.connect('clicked',self._speak_stop,None)
        self.stop_button.set_sensitive(False)
        self.toolbar.insert(Gtk.SeparatorToolItem(),-1)
        
        button=Gtk.ToolButton.new_from_stock(Gtk.STOCK_SELECT_FONT)
        button.connect('clicked',self._select_font,None)
        self.toolbar.insert(button,-1)


        self.translate_button=Gtk.ToolButton.new_from_stock(Gtk.STOCK_NETWORK)
        self.translate_button.set_label('Translator')
        self.toolbar.insert(self.translate_button,-1)
        self.translate_button.connect('clicked',self._translate,None)
        
        
        #voice selection
        self.voicebox=Gtk.Box(Gtk.Orientation.HORIZONTAL)
        frame.pack_start(self.voicebox,0,0,2)
        self.voicecombo=Gtk.ComboBoxText()
        self.voicebox.pack_start(self.voicecombo,0,0,2)
        self.voices=self.core.get_voices()
        n=0
        for i,vox in enumerate(self.voices):
            self.voicecombo.append_text(vox[0]+' ('+vox[1]+')')
            if vox[0] == params['voice']:
                n=i
        self.voicecombo.set_active(n)
        
        self.voicespeed=Gtk.SpinButton.new_with_range(0.5,2.0,0.05)
        self.voicespeed.set_value(params['speed'] or 1.0)
        self.voicebox.pack_start(self.voicespeed,0,0,2)
        

        
        self.resize(900,600)
        
        self.buffer=GtkSource.Buffer()
        self.view=GtkSource.View.new_with_buffer(self.buffer)
        self.view.set_show_line_numbers(True)
        self.view.set_highlight_current_line(True)
        self.view.set_size_request(750, 240);
        self.view.set_left_margin(5)
        self.view.set_right_margin(5)
        self.view.set_indent(15)
        self.view.set_editable(False)
        self.view.set_pixels_below_lines(4)
        self.view.set_wrap_mode(Gtk.WrapMode.WORD_CHAR)
        self.view.set_justification(Gtk.Justification.FILL)
        
        self.speaktag=self.buffer.create_tag('speaking')
        self.speaktag.set_property('background','yellow')
        self.speaktag.set_property('foreground','black')

        
        sw=Gtk.ScrolledWindow()
        sw.add(self.view)
        
        
        frame.pack_start(sw,True,True,0)

        self._apply_font()
        self.show_all()
        self.view.grab_focus()
        self._cursor=0
        GObject.threads_init()
        
    def _init_cursor(self, *args):
        itr=self.buffer.get_iter_at_offset(self._cursor)
        self.view.scroll_to_iter(itr,0.05,True,0.1,0.0)
        self.buffer.place_cursor(itr)
        return False
